/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/rbac/v3/rbac.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPBDEFS_H_
#define ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init envoy_config_rbac_v3_rbac_proto_upbdefinit;

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_RBAC_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.RBAC");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_RBAC_PoliciesEntry_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.RBAC.PoliciesEntry");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Policy_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Policy");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Permission_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Permission");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Permission_Set_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Permission.Set");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Principal_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Principal");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Principal_Set_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Principal.Set");
}

UPB_INLINE const upb_msgdef *envoy_config_rbac_v3_Principal_Authenticated_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_config_rbac_v3_rbac_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.config.rbac.v3.Principal.Authenticated");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPBDEFS_H_ */
