/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     src/proto/grpc/auth/v1/authz_policy.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "src/proto/grpc/auth/v1/authz_policy.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout_field grpc_auth_v1_Peer__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 3},
};

const upb_msglayout grpc_auth_v1_Peer_msginit = {
  NULL,
  &grpc_auth_v1_Peer__fields[0],
  UPB_SIZE(8, 8), 1, false, 255,
};

static const upb_msglayout_field grpc_auth_v1_Header__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 1},
  {2, UPB_SIZE(8, 16), 0, 0, 9, 3},
};

const upb_msglayout grpc_auth_v1_Header_msginit = {
  NULL,
  &grpc_auth_v1_Header__fields[0],
  UPB_SIZE(16, 32), 2, false, 255,
};

static const upb_msglayout *const grpc_auth_v1_Request_submsgs[1] = {
  &grpc_auth_v1_Header_msginit,
};

static const upb_msglayout_field grpc_auth_v1_Request__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 3},
  {3, UPB_SIZE(4, 8), 0, 0, 11, 3},
};

const upb_msglayout grpc_auth_v1_Request_msginit = {
  &grpc_auth_v1_Request_submsgs[0],
  &grpc_auth_v1_Request__fields[0],
  UPB_SIZE(8, 16), 2, false, 255,
};

static const upb_msglayout *const grpc_auth_v1_Rule_submsgs[2] = {
  &grpc_auth_v1_Peer_msginit,
  &grpc_auth_v1_Request_msginit,
};

static const upb_msglayout_field grpc_auth_v1_Rule__fields[3] = {
  {1, UPB_SIZE(4, 8), 0, 0, 9, 1},
  {2, UPB_SIZE(12, 24), 1, 0, 11, 1},
  {3, UPB_SIZE(16, 32), 2, 1, 11, 1},
};

const upb_msglayout grpc_auth_v1_Rule_msginit = {
  &grpc_auth_v1_Rule_submsgs[0],
  &grpc_auth_v1_Rule__fields[0],
  UPB_SIZE(24, 48), 3, false, 255,
};

static const upb_msglayout *const grpc_auth_v1_AuthorizationPolicy_submsgs[1] = {
  &grpc_auth_v1_Rule_msginit,
};

static const upb_msglayout_field grpc_auth_v1_AuthorizationPolicy__fields[3] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 1},
  {2, UPB_SIZE(8, 16), 0, 0, 11, 3},
  {3, UPB_SIZE(12, 24), 0, 0, 11, 3},
};

const upb_msglayout grpc_auth_v1_AuthorizationPolicy_msginit = {
  &grpc_auth_v1_AuthorizationPolicy_submsgs[0],
  &grpc_auth_v1_AuthorizationPolicy__fields[0],
  UPB_SIZE(16, 32), 3, false, 255,
};

#include "upb/port_undef.inc"

