/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/core/v3/substitution_format_string.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "envoy/config/core/v3/substitution_format_string.upb.h"
#include "envoy/config/core/v3/base.upb.h"
#include "envoy/config/core/v3/extension.upb.h"
#include "google/protobuf/struct.upb.h"
#include "udpa/annotations/status.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_config_core_v3_SubstitutionFormatString_submsgs[3] = {
  &envoy_config_core_v3_DataSource_msginit,
  &envoy_config_core_v3_TypedExtensionConfig_msginit,
  &google_protobuf_Struct_msginit,
};

static const upb_msglayout_field envoy_config_core_v3_SubstitutionFormatString__fields[6] = {
  {1, UPB_SIZE(16, 32), UPB_SIZE(-25, -49), 0, 9, 1},
  {2, UPB_SIZE(16, 32), UPB_SIZE(-25, -49), 2, 11, 1},
  {3, UPB_SIZE(0, 0), 0, 0, 8, 1},
  {4, UPB_SIZE(4, 8), 0, 0, 9, 1},
  {5, UPB_SIZE(16, 32), UPB_SIZE(-25, -49), 0, 11, 1},
  {6, UPB_SIZE(12, 24), 0, 1, 11, 3},
};

const upb_msglayout envoy_config_core_v3_SubstitutionFormatString_msginit = {
  &envoy_config_core_v3_SubstitutionFormatString_submsgs[0],
  &envoy_config_core_v3_SubstitutionFormatString__fields[0],
  UPB_SIZE(32, 64), 6, false, 255,
};

#include "upb/port_undef.inc"

