/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     udpa/annotations/security.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "udpa/annotations/security.upb.h"
#include "udpa/annotations/status.upb.h"
#include "google/protobuf/any.upb.h"
#include "google/protobuf/descriptor.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout_field udpa_annotations_FieldSecurityAnnotation__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 8, 1},
  {2, UPB_SIZE(1, 1), 0, 0, 8, 1},
};

const upb_msglayout udpa_annotations_FieldSecurityAnnotation_msginit = {
  NULL,
  &udpa_annotations_FieldSecurityAnnotation__fields[0],
  UPB_SIZE(8, 8), 2, false, 255,
};

#include "upb/port_undef.inc"

